@@ -58,12 +58,15 @@ import java.util.concurrent.ConcurrentHashMap;
 import javax.mail.AuthenticationFailedException;
 import javax.mail.MessagingException;
 import com.openexchange.groupware.AbstractOXException;
+import com.openexchange.imap.acl.ACLExtension;
+import com.openexchange.imap.acl.ACLExtensionFactory;
 import com.openexchange.imap.acl.ACLExtensionInit;
 import com.openexchange.imap.cache.MBoxEnabledCache;
 import com.openexchange.imap.config.IIMAPProperties;
 import com.openexchange.imap.config.IMAPConfig;
 import com.openexchange.imap.config.IMAPSessionProperties;
 import com.openexchange.imap.config.MailAccountIMAPProperties;
+import com.openexchange.imap.converters.IMAPFolderConverter;
 import com.openexchange.imap.entity2acl.Entity2ACLException;
 import com.openexchange.imap.entity2acl.Entity2ACLInit;
 import com.openexchange.imap.ping.IMAPCapabilityAndGreetingCache;
@@ -74,6 +77,7 @@ import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.api.MailLogicTools;
+import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.MIMESessionPropertyNames;
 import com.openexchange.mailaccount.MailAccountException;
@@ -82,6 +86,7 @@ import com.openexchange.monitoring.MonitoringInfo;
 import com.openexchange.server.ServiceException;
 import com.openexchange.session.Session;
 import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
+import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPStore;
 
 /**
@@ -242,6 +247,67 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         }
     }
 
+    @Override
+    public int getUnreadMessagesCount(final String fullname) throws MailException {
+        if (!isConnected()) {
+            connect(false);
+        }
+        /*
+         * Check for root folder
+         */
+        if (MailFolder.DEFAULT_FOLDER_ID.equals(fullname)) {
+            return 0;
+        }
+        try {
+            /*
+             * Obtain IMAP folder
+             */
+            final IMAPFolder imapFolder = (IMAPFolder) imapStore.getFolder(fullname);
+            final boolean exists = imapFolder.exists();
+            final IMAPConfig imapConfig = getIMAPConfig();
+            if (!exists) {
+                throw IMAPException.create(IMAPException.Code.FOLDER_NOT_FOUND, imapConfig, session, fullname);
+            }
+            String[] attrs;
+            try {
+                attrs = imapFolder.getAttributes();
+            } catch (final NullPointerException e) {
+                /*
+                 * No attributes available.
+                 */
+                attrs = null;
+            }
+            if (null != attrs) {
+                for (final String attribute : attrs) {
+                    if ("\\NonExistent".equalsIgnoreCase(attribute)) {
+                        throw IMAPException.create(IMAPException.Code.FOLDER_NOT_FOUND, imapConfig, session, fullname);
+                    }
+                }
+            }
+            final int retval;
+            /*
+             * Selectable?
+             */
+            if ((imapFolder.getType() & javax.mail.Folder.HOLDS_MESSAGES) > 0) {
+                /*
+                 * Check read access
+                 */
+                final ACLExtension aclExtension = ACLExtensionFactory.getInstance().getACLExtension(imapConfig);
+                if (!aclExtension.aclSupport() || aclExtension.canRead(IMAPFolderConverter.getOwnRights(imapFolder, session, imapConfig))) {
+                    retval = IMAPFolderConverter.getUnreadCount(imapFolder);
+                } else {
+                    // ACL support AND no read access
+                    retval = -1;
+                }
+            } else {
+                retval = -1;
+            }
+            return retval;
+        } catch (final MessagingException e) {
+            throw MIMEMailException.handleMessagingException(e, getMailConfig(), session);
+        }
+    }
+
     @Override
     public boolean ping() throws MailException {
         final IMAPConfig config = getIMAPConfig();
@@ -269,13 +335,12 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             /*
              * Get parameterized IMAP session
              */
-            final javax.mail.Session imapSession =
-                setConnectProperties(
-                    config.getPort(),
-                    config.isSecure(),
-                    imapConfProps.getImapTimeout(),
-                    imapConfProps.getImapConnectionTimeout(),
-                    imapProps);
+            final javax.mail.Session imapSession = setConnectProperties(
+                config.getPort(),
+                config.isSecure(),
+                imapConfProps.getImapTimeout(),
+                imapConfProps.getImapConnectionTimeout(),
+                imapProps);
             /*
              * Check if debug should be enabled
              */
@@ -351,13 +416,12 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             /*
              * Get parameterized IMAP session
              */
-            imapSession =
-                setConnectProperties(
-                    config.getPort(),
-                    config.isSecure(),
-                    imapConfProps.getImapTimeout(),
-                    imapConfProps.getImapConnectionTimeout(),
-                    imapProps);
+            imapSession = setConnectProperties(
+                config.getPort(),
+                config.isSecure(),
+                imapConfProps.getImapTimeout(),
+                imapConfProps.getImapConnectionTimeout(),
+                imapProps);
             /*
              * Check if debug should be enabled
              */
@@ -736,6 +800,9 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             // Security.setProperty("ssl.SocketFactory.provider", socketFactoryClass);
         }
+        /*
+         * Create new IMAP session from initialized properties
+         */
         return javax.mail.Session.getInstance(imapProps, null);
     }
 
