@@ -72,6 +72,7 @@ import javax.mail.MessagingException;
 import javax.mail.StoreClosedException;
 import javax.mail.internet.MimeMessage;
 import com.openexchange.groupware.ldap.UserException;
+import com.openexchange.imap.AllFetch.LowCostItem;
 import com.openexchange.imap.cache.RightsCache;
 import com.openexchange.imap.cache.UserFlagsCache;
 import com.openexchange.imap.command.CopyIMAPCommand;
@@ -128,6 +129,8 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
 
     private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(IMAPMessageStorage.class);
 
+    private static final boolean DEBUG = LOG.isDebugEnabled();
+
     /**
      * Serial version UID
      */
@@ -172,9 +175,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
     private MailAccount getMailAccount() throws MailException {
         if (mailAccount == null) {
             try {
-                final MailAccountStorageService storageService = IMAPServiceRegistry.getService(
-                    MailAccountStorageService.class,
-                    true);
+                final MailAccountStorageService storageService = IMAPServiceRegistry.getService(MailAccountStorageService.class, true);
                 mailAccount = storageService.getMailAccount(accountId, session.getUserId(), session.getContextId());
             } catch (final ServiceException e) {
                 throw new MailException(e);
@@ -276,7 +277,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
         } catch (final StoreClosedException e) {
             throw MIMEMailException.handleMessagingException(e, imapConfig, session);
         } catch (final MessagingException e) {
-            if (LOG.isDebugEnabled()) {
+            if (DEBUG) {
                 LOG.debug("Fetch with BODYSTRUCTURE failed.", e);
             }
             fetchValidSeqNums(lastPos, len, seqNums, messages, fetchProfile, isRev1, body, true);
@@ -289,21 +290,17 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
         final long start = System.currentTimeMillis();
         final Message[] submessages;
         if (ignoreBodystructure) {
-            submessages = new FetchIMAPCommand(
-                imapFolder,
-                isRev1,
-                subarr,
-                FetchIMAPCommand.getSafeFetchProfile(fetchProfile),
-                false,
-                true,
-                body).setDetermineAttachmentyHeader(true).doCommand();
+            submessages =
+                new FetchIMAPCommand(imapFolder, isRev1, subarr, FetchIMAPCommand.getSafeFetchProfile(fetchProfile), false, true, body).setDetermineAttachmentyHeader(
+                    true).doCommand();
         } else {
             submessages = new FetchIMAPCommand(imapFolder, isRev1, subarr, fetchProfile, false, true, body).doCommand();
         }
-        mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
-        if (LOG.isDebugEnabled()) {
-            LOG.debug(new StringBuilder(128).append("IMAP fetch for ").append(subarr.length).append(" messages took ").append(
-                (System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+        final long time = System.currentTimeMillis() - start;
+        mailInterfaceMonitor.addUseTime(time);
+        if (DEBUG) {
+            LOG.debug(new StringBuilder(128).append("IMAP fetch for ").append(subarr.length).append(" messages took ").append(time).append(
+                STR_MSEC).toString());
         }
         System.arraycopy(submessages, 0, messages, lastPos, submessages.length);
     }
@@ -324,6 +321,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                 // .toString());
                 return null;
             }
+            // TODO: Examine behavior when applying: msg.setPeek(!markSeen);
             final MailMessage mail;
             try {
                 mail = MIMEMessageConverter.convertMessage(msg);
@@ -438,10 +436,8 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                     msgs = new Message[retvalLength];
                     System.arraycopy(tmp, fromIndex, msgs, 0, retvalLength);
                 }
-                mails = convert2Mails(
-                    msgs,
-                    usedFields.toArray(),
-                    usedFields.contains(MailField.BODY) || usedFields.contains(MailField.FULL));
+                mails =
+                    convert2Mails(msgs, usedFields.toArray(), usedFields.contains(MailField.BODY) || usedFields.contains(MailField.FULL));
                 if (usedFields.contains(MailField.ACCOUNT_NAME) || usedFields.contains(MailField.FULL)) {
                     setAccountInfo(mails);
                 }
@@ -452,22 +448,40 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                 final int size = filter == null ? imapFolder.getMessageCount() : filter.length;
                 final FetchProfile fetchProfile = getFetchProfile(usedFields.toArray(), getIMAPProperties().isFastFetch());
                 final boolean body = usedFields.contains(MailField.BODY) || usedFields.contains(MailField.FULL);
-                final long start = System.currentTimeMillis();
-                if (filter == null) {
-                    msgs = new FetchIMAPCommand(imapFolder, imapConfig.getImapCapabilities().hasIMAP4rev1(), fetchProfile, size, body).doCommand();
+                if (DEBUG) {
+                    final long start = System.currentTimeMillis();
+                    if (filter == null) {
+                        msgs =
+                            new FetchIMAPCommand(imapFolder, imapConfig.getImapCapabilities().hasIMAP4rev1(), fetchProfile, size, body).doCommand();
+                    } else {
+                        msgs =
+                            new FetchIMAPCommand(
+                                imapFolder,
+                                imapConfig.getImapCapabilities().hasIMAP4rev1(),
+                                filter,
+                                fetchProfile,
+                                false,
+                                false,
+                                body).doCommand();
+                    }
+                    final long time = System.currentTimeMillis() - start;
+                    LOG.debug(new StringBuilder(128).append("IMAP fetch for ").append(size).append(" messages took ").append(time).append(
+                        "msec").toString());
                 } else {
-                    msgs = new FetchIMAPCommand(
-                        imapFolder,
-                        imapConfig.getImapCapabilities().hasIMAP4rev1(),
-                        filter,
-                        fetchProfile,
-                        false,
-                        false,
-                        body).doCommand();
-                }
-                if (LOG.isDebugEnabled()) {
-                    LOG.debug(new StringBuilder(128).append("IMAP fetch for ").append(size).append(" messages took ").append(
-                        (System.currentTimeMillis() - start)).append("msec").toString());
+                    if (filter == null) {
+                        msgs =
+                            new FetchIMAPCommand(imapFolder, imapConfig.getImapCapabilities().hasIMAP4rev1(), fetchProfile, size, body).doCommand();
+                    } else {
+                        msgs =
+                            new FetchIMAPCommand(
+                                imapFolder,
+                                imapConfig.getImapCapabilities().hasIMAP4rev1(),
+                                filter,
+                                fetchProfile,
+                                false,
+                                false,
+                                body).doCommand();
+                    }
                 }
                 if ((msgs == null) || (msgs.length == 0)) {
                     return new MailMessage[0];
@@ -583,14 +597,8 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
             usedFields.add(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             final FetchProfile fetchProfile = getFetchProfile(usedFields.toArray(), getIMAPProperties().isFastFetch());
             final boolean body = usedFields.contains(MailField.BODY) || usedFields.contains(MailField.FULL);
-            Message[] msgs = new FetchIMAPCommand(
-                imapFolder,
-                imapConfig.getImapCapabilities().hasIMAP4rev1(),
-                seqnums,
-                fetchProfile,
-                false,
-                true,
-                body).doCommand();
+            Message[] msgs =
+                new FetchIMAPCommand(imapFolder, imapConfig.getImapCapabilities().hasIMAP4rev1(), seqnums, fetchProfile, false, true, body).doCommand();
             /*
              * Apply thread level
              */
@@ -666,7 +674,8 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                  * Get ( & fetch) new messages
                  */
                 final long start = System.currentTimeMillis();
-                final Message[] msgs = IMAPCommandsCollection.getUnreadMessages(imapFolder, fields, sortField, getIMAPProperties());
+                final Message[] msgs =
+                    IMAPCommandsCollection.getUnreadMessages(imapFolder, fields, sortField, getIMAPProperties().isFastFetch());
                 mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
                 if ((msgs == null) || (msgs.length == 0) || limit == 0) {
                     return EMPTY_RETVAL;
@@ -743,7 +752,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
             // Nothing to do on empty ID array
             return;
         }
-        final StringBuilder debug = LOG.isDebugEnabled() ? new StringBuilder(128) : null;
+        final StringBuilder debug = DEBUG ? new StringBuilder(128) : null;
         final long[] remain;
         final int blockSize = getIMAPProperties().getBlockSize();
         if (blockSize > 0 && msgUIDs.length > blockSize) {
@@ -776,12 +785,15 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
              * Copy messages to folder "TRASH"
              */
             try {
-                final long start = System.currentTimeMillis();
-                new CopyIMAPCommand(imapFolder, uids, trashFullname, false, true).doCommand();
-                if (LOG.isDebugEnabled()) {
+                if (DEBUG) {
+                    final long start = System.currentTimeMillis();
+                    new CopyIMAPCommand(imapFolder, uids, trashFullname, false, true).doCommand();
+                    final long time = System.currentTimeMillis() - start;
                     sb.setLength(0);
                     LOG.debug(sb.append("\"Soft Delete\": ").append(uids.length).append(" messages copied to default trash folder \"").append(
-                        trashFullname).append("\" in ").append((System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+                        trashFullname).append("\" in ").append(time).append(STR_MSEC).toString());
+                } else {
+                    new CopyIMAPCommand(imapFolder, uids, trashFullname, false, true).doCommand();
                 }
             } catch (final MessagingException e) {
                 if (e.getMessage().indexOf("Over quota") > -1) {
@@ -803,12 +815,15 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
         /*
          * Mark messages as \DELETED...
          */
-        final long start = System.currentTimeMillis();
-        new FlagsIMAPCommand(imapFolder, uids, FLAGS_DELETED, true, true, false).doCommand();
-        if (LOG.isDebugEnabled()) {
+        if (DEBUG) {
+            final long start = System.currentTimeMillis();
+            new FlagsIMAPCommand(imapFolder, uids, FLAGS_DELETED, true, true, false).doCommand();
+            final long dur = System.currentTimeMillis() - start;
             sb.setLength(0);
-            LOG.debug(sb.append(uids.length).append(" messages marked as deleted (through system flag \\DELETED) in ").append(
-                (System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+            LOG.debug(sb.append(uids.length).append(" messages marked as deleted (through system flag \\DELETED) in ").append(dur).append(
+                STR_MSEC).toString());
+        } else {
+            new FlagsIMAPCommand(imapFolder, uids, FLAGS_DELETED, true, true, false).doCommand();
         }
         /*
          * ... and perform EXPUNGE
@@ -933,13 +948,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
              */
             final long[] result = new long[mailIds.length];
             final int blockSize = getIMAPProperties().getBlockSize();
-            final StringBuilder debug;
-            if (LOG.isDebugEnabled()) {
-                debug = new StringBuilder(128);
-            } else {
-                debug = null;
-            }
-
+            final StringBuilder debug = DEBUG ? new StringBuilder(128) : null;
             int offset = 0;
             final long[] remain;
             if (blockSize > 0 && mailIds.length > blockSize) {
@@ -978,12 +987,15 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                 final IMAPFolder destFolder = setAndOpenFolder(destFullname, Folder.READ_WRITE);
                 try {
                     if (destFolder.getMessageCount() > 0) {
-                        final long start = System.currentTimeMillis();
-                        new FlagsIMAPCommand(destFolder, FLAGS_DRAFT, true, true).doCommand();
-                        if (LOG.isDebugEnabled()) {
+                        if (DEBUG) {
+                            final long start = System.currentTimeMillis();
+                            new FlagsIMAPCommand(destFolder, FLAGS_DRAFT, true, true).doCommand();
+                            final long time = System.currentTimeMillis() - start;
                             LOG.debug(new StringBuilder(128).append(
                                 "A copy/move to default drafts folder => All messages' \\Draft flag in ").append(destFullname).append(
-                                " set in ").append((System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+                                " set in ").append(time).append(STR_MSEC).toString());
+                        } else {
+                            new FlagsIMAPCommand(destFolder, FLAGS_DRAFT, true, true).doCommand();
                         }
                     }
                 } finally {
@@ -995,11 +1007,14 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                  */
                 final IMAPFolder destFolder = setAndOpenFolder(destFullname, Folder.READ_WRITE);
                 try {
-                    final long start = System.currentTimeMillis();
-                    new FlagsIMAPCommand(destFolder, FLAGS_DRAFT, false, true).doCommand();
-                    if (LOG.isDebugEnabled()) {
+                    if (DEBUG) {
+                        final long start = System.currentTimeMillis();
+                        new FlagsIMAPCommand(destFolder, FLAGS_DRAFT, false, true).doCommand();
+                        final long time = System.currentTimeMillis() - start;
                         LOG.debug(new StringBuilder(128).append("A copy/move from default drafts folder => All messages' \\Draft flag in ").append(
-                            destFullname).append(" unset in ").append((System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+                            destFullname).append(" unset in ").append(time).append(STR_MSEC).toString());
+                    } else {
+                        new FlagsIMAPCommand(destFolder, FLAGS_DRAFT, false, true).doCommand();
                     }
                 } finally {
                     destFolder.close(false);
@@ -1015,11 +1030,15 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
     }
 
     private long[] copyOrMoveByUID(final boolean move, final boolean fast, final String destFullname, final long[] tmp, final StringBuilder sb) throws MessagingException, MailException, IMAPException {
-        long start = System.currentTimeMillis();
-        long[] uids = new CopyIMAPCommand(imapFolder, tmp, destFullname, false, fast).doCommand();
-        if (LOG.isDebugEnabled()) {
+        long[] uids;
+        if (DEBUG) {
+            final long start = System.currentTimeMillis();
+            uids = new CopyIMAPCommand(imapFolder, tmp, destFullname, false, fast).doCommand();
+            final long time = System.currentTimeMillis() - start;
             sb.setLength(0);
-            LOG.debug(sb.append(tmp.length).append(" messages copied in ").append((System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+            LOG.debug(sb.append(tmp.length).append(" messages copied in ").append(time).append(STR_MSEC).toString());
+        } else {
+            uids = new CopyIMAPCommand(imapFolder, tmp, destFullname, false, fast).doCommand();
         }
         if (!fast && ((uids == null) || noUIDsAssigned(uids, tmp.length))) {
             /*
@@ -1028,12 +1047,15 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
             uids = getDestinationUIDs(tmp, destFullname);
         }
         if (move) {
-            start = System.currentTimeMillis();
-            new FlagsIMAPCommand(imapFolder, tmp, FLAGS_DELETED, true, true, false).doCommand();
-            if (LOG.isDebugEnabled()) {
+            if (DEBUG) {
+                final long start = System.currentTimeMillis();
+                new FlagsIMAPCommand(imapFolder, tmp, FLAGS_DELETED, true, true, false).doCommand();
+                final long time = System.currentTimeMillis() - start;
                 sb.setLength(0);
-                LOG.debug(sb.append(tmp.length).append(" messages marked as expunged (through system flag \\DELETED) in ").append(
-                    (System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+                LOG.debug(sb.append(tmp.length).append(" messages marked as expunged (through system flag \\DELETED) in ").append(time).append(
+                    STR_MSEC).toString());
+            } else {
+                new FlagsIMAPCommand(imapFolder, tmp, FLAGS_DELETED, true, true, false).doCommand();
             }
             try {
                 IMAPCommandsCollection.uidExpungeWithFallback(imapFolder, tmp, imapConfig.getImapCapabilities().hasUIDPlus());
@@ -1201,7 +1223,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
             notifyIMAPFolderModification(destFullname);
             return retval;
         } catch (final MessagingException e) {
-            if (LOG.isDebugEnabled()) {
+            if (DEBUG) {
                 final Exception next = e.getNextException();
                 if (next instanceof CommandFailedException) {
                     final StringBuilder sb = new StringBuilder(8192);
@@ -1290,7 +1312,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                     }
                     affectedFlags.add(MailMessage.USER_FORWARDED);
                     applyFlags = true;
-                } else if (LOG.isDebugEnabled()) {
+                } else if (DEBUG) {
                     LOG.debug(new StringBuilder().append("IMAP server ").append(imapConfig.getImapServerSocketAddress()).append(
                         " does not support user flags. Skipping forwarded flag."));
                 }
@@ -1308,17 +1330,20 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                     }
                     affectedFlags.add(MailMessage.USER_READ_ACK);
                     applyFlags = true;
-                } else if (LOG.isDebugEnabled()) {
+                } else if (DEBUG) {
                     LOG.debug(new StringBuilder().append("IMAP server ").append(imapConfig.getImapServerSocketAddress()).append(
                         " does not support user flags. Skipping read-ack flag."));
                 }
             }
             if (applyFlags) {
-                final long start = System.currentTimeMillis();
-                new FlagsIMAPCommand(imapFolder, msgUIDs, affectedFlags, set, true, false).doCommand();
-                if (LOG.isDebugEnabled()) {
-                    LOG.debug(new StringBuilder(128).append("Flags applied to ").append(msgUIDs.length).append(" messages in ").append(
-                        (System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+                if (DEBUG) {
+                    final long start = System.currentTimeMillis();
+                    new FlagsIMAPCommand(imapFolder, msgUIDs, affectedFlags, set, true, false).doCommand();
+                    final long time = System.currentTimeMillis() - start;
+                    LOG.debug(new StringBuilder(128).append("Flags applied to ").append(msgUIDs.length).append(" messages in ").append(time).append(
+                        STR_MSEC).toString());
+                } else {
+                    new FlagsIMAPCommand(imapFolder, msgUIDs, affectedFlags, set, true, false).doCommand();
                 }
             }
             /*
@@ -1349,7 +1374,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                 /*
                  * User flags are disabled
                  */
-                if (LOG.isDebugEnabled()) {
+                if (DEBUG) {
                     LOG.debug("User flags are disabled or not supported. Update of color flag ignored.");
                 }
                 return;
@@ -1380,14 +1405,14 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
             long start = System.currentTimeMillis();
             IMAPCommandsCollection.clearAllColorLabels(imapFolder, msgUIDs);
             mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
-            if (LOG.isDebugEnabled()) {
+            if (DEBUG) {
                 LOG.debug(new StringBuilder(128).append("All color flags cleared from ").append(msgUIDs.length).append(" messages in ").append(
                     (System.currentTimeMillis() - start)).append(STR_MSEC).toString());
             }
             start = System.currentTimeMillis();
             IMAPCommandsCollection.setColorLabel(imapFolder, msgUIDs, MailMessage.getColorLabelStringValue(colorLabel));
             mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
-            if (LOG.isDebugEnabled()) {
+            if (DEBUG) {
                 LOG.debug(new StringBuilder(128).append("All color flags set in ").append(msgUIDs.length).append(" messages in ").append(
                     (System.currentTimeMillis() - start)).append(STR_MSEC).toString());
             }
@@ -1519,13 +1544,14 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
          */
         MailMessage[] retval;
         {
-            final String lowCostItems = getFetchCommand(lowCostFields);
+            final LowCostItem[] lowCostItems = getLowCostItems(lowCostFields);
             final long start = System.currentTimeMillis();
-            retval = AllFetch.fetchLowCost(imapFolder, lowCostItems, OrderDirection.ASC.equals(order));
+            retval = AllFetch.fetchLowCost(imapFolder, lowCostItems, OrderDirection.ASC.equals(order), imapConfig, session);
             mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
-            if (LOG.isDebugEnabled()) {
-                LOG.debug(new StringBuilder(128).append(fullname).append(": IMAP all fetch >>>FETCH 1:* (").append(lowCostItems).append(
-                    ")<<< took ").append((System.currentTimeMillis() - start)).append(STR_MSEC).toString());
+            if (DEBUG) {
+                LOG.debug(new StringBuilder(128).append(fullname).append(": IMAP all fetch >>>FETCH 1:* (").append(
+                    AllFetch.getFetchCommand(lowCostItems)).append(")<<< took ").append((System.currentTimeMillis() - start)).append(
+                    STR_MSEC).toString(), new Throwable());
             }
         }
         if (retval == null || retval.length == 0) {
@@ -1554,34 +1580,48 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
         return retval;
     }
 
-    private static String getFetchCommand(final MailFields fields) {
-        final StringBuilder command = new StringBuilder(128);
+    private static final MailFields FIELDS_ENV =
+        new MailFields(
+            new MailField[] { MailField.SENT_DATE, MailField.FROM, MailField.TO, MailField.CC, MailField.BCC, MailField.SUBJECT });
+
+    private static LowCostItem[] getLowCostItems(final MailFields fields) {
+        final Set<LowCostItem> l = EnumSet.noneOf(LowCostItem.class);
         if (fields.contains(MailField.RECEIVED_DATE)) {
-            command.append("INTERNALDATE ");
+            l.add(LowCostItem.INTERNALDATE);
         }
         if (fields.contains(MailField.ID)) {
-            command.append("UID ");
+            l.add(LowCostItem.UID);
         }
         if (fields.contains(MailField.FLAGS) || fields.contains(MailField.COLOR_LABEL)) {
-            command.append("FLAGS ");
+            l.add(LowCostItem.FLAGS);
         }
         if (fields.contains(MailField.CONTENT_TYPE)) {
-            command.append("BODYSTRUCTURE ");
+            l.add(LowCostItem.BODYSTRUCTURE);
         }
         if (fields.contains(MailField.SIZE)) {
-            command.append("RFC822.SIZE ");
+            l.add(LowCostItem.SIZE);
         }
-        return command.deleteCharAt(command.length() - 1).toString();
+        if (fields.containsAny(FIELDS_ENV)) {
+            l.add(LowCostItem.ENVELOPE);
+        }
+        return l.toArray(new LowCostItem[l.size()]);
     }
 
-    private static final EnumSet<MailField> LOW_COST = EnumSet.of(
-        MailField.ID,
-        MailField.FOLDER_ID,
-        MailField.RECEIVED_DATE,
-        MailField.FLAGS,
-        MailField.COLOR_LABEL,
-        MailField.SIZE,
-        MailField.CONTENT_TYPE);
+    private static final EnumSet<MailField> LOW_COST =
+        EnumSet.of(
+            MailField.ID,
+            MailField.FOLDER_ID,
+            MailField.RECEIVED_DATE,
+            MailField.FLAGS,
+            MailField.COLOR_LABEL,
+            MailField.SIZE,
+            MailField.CONTENT_TYPE,
+            MailField.SENT_DATE,
+            MailField.FROM,
+            MailField.TO,
+            MailField.CC,
+            MailField.BCC,
+            MailField.SUBJECT);
 
     private static boolean onlyLowCostFields(final MailFields fields) {
         final Set<MailField> set = fields.toSet();
@@ -1690,6 +1730,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
                  */
                 try {
                     SpamHandlerRegistry.getSpamHandlerBySession(session, accountId, IMAPProvider.getInstance()).handleSpam(
+                        accountId,
                         imapFolder.getFullName(),
                         longs2uids(msgUIDs),
                         move,
@@ -1714,7 +1755,12 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
              * Handle ham.
              */
             try {
-                IMAPProvider.getInstance().getSpamHandler().handleHam(imapFolder.getFullName(), longs2uids(msgUIDs), move, session);
+                IMAPProvider.getInstance().getSpamHandler().handleHam(
+                    accountId,
+                    imapFolder.getFullName(),
+                    longs2uids(msgUIDs),
+                    move,
+                    session);
                 /*
                  * Close and reopen to force internal message cache update
                  */
@@ -1791,6 +1837,9 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
      * @throws MailException If mail account cannot be obtained
      */
     private MailMessage setAccountInfo(final MailMessage mailMessage) throws MailException {
+        if (null == mailMessage) {
+            return null;
+        }
         final MailAccount account = getMailAccount();
         final String name = account.getName();
         final int id = account.getId();
@@ -1812,8 +1861,10 @@ public final class IMAPMessageStorage extends IMAPFolderWorker {
         final int id = account.getId();
         for (int i = 0; i < mailMessages.length; i++) {
             final MailMessage mailMessage = mailMessages[i];
-            mailMessage.setAccountId(id);
-            mailMessage.setAccountName(name);
+            if (null != mailMessage) {
+                mailMessage.setAccountId(id);
+                mailMessage.setAccountName(name);
+            }
         }
         return mailMessages;
     }
