@@ -484,7 +484,7 @@ public final class FetchIMAPCommand extends AbstractIMAPCommand<Message[]> {
             if (determineAttachmentByHeader) {
                 final String cts = msg.getHeader(MessageHeaders.HDR_CONTENT_TYPE, null);
                 if (null != cts) {
-                    msg.setHasAttachment(new ContentType(cts).isMimeType("multipart/mixed"));
+                    msg.setHasAttachment(new ContentType(cts).startsWith("multipart/mixed"));
                 }
             }
         } catch (final MessagingException e) {
@@ -710,9 +710,18 @@ public final class FetchIMAPCommand extends AbstractIMAPCommand<Message[]> {
             msg.setReplyTo(env.replyTo);
             msg.setHeader(MessageHeaders.HDR_IN_REPLY_TO, env.inReplyTo);
             msg.setHeader(MessageHeaders.HDR_MESSAGE_ID, env.messageId);
-
+            /*-
+             * Hmm... Why does ENVELOPE FETCH response omit CR?LFs in subject?!
+             * 
+             * Example:
+             * Subject: =?UTF-8?Q?Nur_noch_kurze_Zeit:_1_Freimona?=
+             *  =?UTF-8?Q?t_f=C3=BCr_3_erfolgreiche_Einladungen?=
+             *  
+             * is transferred as:
+             * =?UTF-8?Q?Nur_noch_kurze_Zeit:_1_Freimona?= =?UTF-8?Q?t_f=C3=BCr_3_erfolgreiche_Einladungen?=
+             */
             final String subject;
-            if (env.subject == null) {
+            if (null == env.subject) {
                 subject = "";
             } else {
                 final char[] chars = env.subject.toCharArray();
@@ -728,7 +737,7 @@ public final class FetchIMAPCommand extends AbstractIMAPCommand<Message[]> {
                     }
                     i++;
                 }
-                subject = MIMEMessageUtility.decodeMultiEncodedHeader(sb.toString());
+                subject = MIMEMessageUtility.decodeEnvelopeHeader(sb.toString());
             }
             msg.setSubject(subject, MailProperties.getInstance().getDefaultMimeCharset());
 
