@@ -50,13 +50,16 @@
 package com.openexchange.imap.converters;
 
 import java.net.InetSocketAddress;
+import java.util.Locale;
 import javax.mail.Folder;
 import javax.mail.MessagingException;
 import com.openexchange.groupware.AbstractOXException;
+import com.openexchange.groupware.Component;
 import com.openexchange.groupware.EnumComponent;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.groupware.contexts.impl.ContextStorage;
+import com.openexchange.groupware.ldap.LdapException;
 import com.openexchange.imap.ACLPermission;
 import com.openexchange.imap.IMAPCommandsCollection;
 import com.openexchange.imap.IMAPException;
@@ -74,8 +77,10 @@ import com.openexchange.imap.entity2acl.Entity2ACLException;
 import com.openexchange.imap.entity2acl.IMAPServer;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.MailSessionParameterNames;
+import com.openexchange.mail.MailSessionCache;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.dataobjects.MailFolder;
+import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.usersetting.UserSettingMailStorage;
 import com.openexchange.server.impl.OCLPermission;
@@ -134,6 +139,8 @@ public final class IMAPFolderConverter {
 
     private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(IMAPFolderConverter.class);
 
+    private static final boolean DEBUG = LOG.isDebugEnabled();
+
     private static final Rights RIGHTS_EMPTY = new Rights();
 
     /**
@@ -170,6 +177,11 @@ public final class IMAPFolderConverter {
             imapFolder.getSeparator());
     }
 
+    private static final DefaultFolderType[] TYPES =
+        {
+            DefaultFolderType.DRAFTS, DefaultFolderType.SENT, DefaultFolderType.SPAM, DefaultFolderType.TRASH,
+            DefaultFolderType.CONFIRMED_SPAM, DefaultFolderType.CONFIRMED_HAM, DefaultFolderType.INBOX };
+
     /**
      * Creates a folder data object from given IMAP folder.
      * 
@@ -245,17 +257,21 @@ public final class IMAPFolderConverter {
              */
             if ("INBOX".equals(imapFullname)) {
                 mailFolder.setDefaultFolder(true);
+                mailFolder.setDefaultFolderType(DefaultFolderType.INBOX);
             } else if (isDefaultFoldersChecked(session, imapConfig.getAccountId())) {
-                final int len = UserSettingMailStorage.getInstance().getUserSettingMail(
-                    session.getUserId(),
-                    ContextStorage.getStorageContext(session.getContextId())).isSpamEnabled() ? 6 : 4;
+                final int len =
+                    UserSettingMailStorage.getInstance().getUserSettingMail(
+                        session.getUserId(),
+                        ContextStorage.getStorageContext(session.getContextId())).isSpamEnabled() ? 6 : 4;
                 for (int i = 0; (i < len) && !mailFolder.isDefaultFolder(); i++) {
                     if (mailFolder.getFullname().equals(getDefaultMailFolder(i, session, imapConfig.getAccountId()))) {
                         mailFolder.setDefaultFolder(true);
+                        mailFolder.setDefaultFolderType(TYPES[i]);
                     }
                 }
                 if (!mailFolder.containsDefaultFolder()) {
                     mailFolder.setDefaultFolder(false);
+                    mailFolder.setDefaultFolderType(DefaultFolderType.NONE);
                 }
             }
             /*
@@ -309,7 +325,7 @@ public final class IMAPFolderConverter {
                         ownPermission.parseRights((ownRights = (Rights) RIGHTS_EMPTY.clone()), imapConfig);
                     }
                 } else {
-                    ownPermission.parseRights((ownRights = getOwnRightsInternal(imapFolder, session, imapConfig)), imapConfig);
+                    ownPermission.parseRights((ownRights = getOwnRights(imapFolder, session, imapConfig)), imapConfig);
                 }
                 /*
                  * Check own permission against folder type
@@ -339,8 +355,8 @@ public final class IMAPFolderConverter {
                          */
                         final int[] status = IMAPCommandsCollection.getStatus(imapFolder);
                         mailFolder.setMessageCount(status[0]);
-                        mailFolder.setMessageCount(status[1]);
-                        mailFolder.setMessageCount(status[2]);
+                        mailFolder.setNewMessageCount(status[1]);
+                        mailFolder.setUnreadMessageCount(status[2]);
                     } else {
                         // Re-throw
                         throw e;
@@ -393,7 +409,8 @@ public final class IMAPFolderConverter {
         final ListInfo[] li = (ListInfo[]) imapFolder.doCommand(new IMAPFolder.ProtocolCommand() {
 
             public Object doCommand(final IMAPProtocol protocol) throws ProtocolException {
-                final String pattern = mailFolder.isRootFolder() ? "%" : new StringBuilder().append(fullname).append(separator).append('%').toString();
+                final String pattern =
+                    mailFolder.isRootFolder() ? "%" : new StringBuilder().append(fullname).append(separator).append('%').toString();
                 if (checkSubscribed) {
                     return protocol.lsub("", pattern);
                 }
@@ -460,6 +477,7 @@ public final class IMAPFolderConverter {
             mailFolder.setName(MailFolder.DEFAULT_FOLDER_NAME);
             mailFolder.setParentFullname(null);
             mailFolder.setDefaultFolder(false);
+            mailFolder.setDefaultFolderType(DefaultFolderType.NONE);
             /*
              * Root folder only holds folders but no messages
              */
@@ -469,7 +487,8 @@ public final class IMAPFolderConverter {
              * Check if subfolder creation is allowed
              */
             final ACLPermission ownPermission = new ACLPermission();
-            final int fp = RootSubfolderCache.canCreateSubfolders(rootFolder, true, session, imapConfig.getAccountId()).booleanValue() ? OCLPermission.CREATE_SUB_FOLDERS : OCLPermission.READ_FOLDER;
+            final int fp =
+                RootSubfolderCache.canCreateSubfolders(rootFolder, true, session, imapConfig.getAccountId()).booleanValue() ? OCLPermission.CREATE_SUB_FOLDERS : OCLPermission.READ_FOLDER;
             ownPermission.setEntity(session.getUserId());
             ownPermission.setAllPermission(fp, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS);
             ownPermission.setFolderAdmin(false);
@@ -497,12 +516,14 @@ public final class IMAPFolderConverter {
     }
 
     private static boolean isDefaultFoldersChecked(final Session session, final int accountId) {
-        final Boolean b = (Boolean) session.getParameter(MailSessionParameterNames.getParamDefaultFolderChecked(accountId));
+        final Boolean b =
+            MailSessionCache.getInstance(session).getParameter(accountId, MailSessionParameterNames.getParamDefaultFolderChecked());
         return (b != null) && b.booleanValue();
     }
 
     private static String getDefaultMailFolder(final int index, final Session session, final int accountId) {
-        final String[] arr = (String[]) session.getParameter(MailSessionParameterNames.getParamDefaultFolderArray(accountId));
+        final String[] arr =
+            MailSessionCache.getInstance(session).getParameter(accountId, MailSessionParameterNames.getParamDefaultFolderArray());
         return arr == null ? null : arr[index];
     }
 
@@ -536,15 +557,14 @@ public final class IMAPFolderConverter {
             throw MIMEMailException.handleMessagingException(e);
         }
         try {
-            final Entity2ACLArgs args = new Entity2ACLArgsImpl(imapConfig.getAccountId(), new InetSocketAddress(
-                imapConfig.getServer(),
-                imapConfig.getPort()), session.getUserId(), imapFolder.getFullName(), imapFolder.getSeparator());
-            final StringBuilder debugBuilder;
-            if (LOG.isDebugEnabled()) {
-                debugBuilder = new StringBuilder(128);
-            } else {
-                debugBuilder = null;
-            }
+            final Entity2ACLArgs args =
+                new Entity2ACLArgsImpl(
+                    imapConfig.getAccountId(),
+                    new InetSocketAddress(imapConfig.getServer(), imapConfig.getPort()),
+                    session.getUserId(),
+                    imapFolder.getFullName(),
+                    imapFolder.getSeparator());
+            final StringBuilder debugBuilder = DEBUG ? new StringBuilder(128) : null;
             for (int j = 0; j < acls.length; j++) {
                 final ACLPermission aclPerm = new ACLPermission();
                 try {
@@ -552,7 +572,7 @@ public final class IMAPFolderConverter {
                     mailFolder.addPermission(aclPerm);
                 } catch (final AbstractOXException e) {
                     if (isUnknownEntityError(e)) {
-                        if (LOG.isDebugEnabled()) {
+                        if (DEBUG) {
                             debugBuilder.setLength(0);
                             LOG.debug(debugBuilder.append("Cannot map ACL entity named \"").append(acls[j].getName()).append(
                                 "\" to a system user").toString());
@@ -601,7 +621,9 @@ public final class IMAPFolderConverter {
     }
 
     private static boolean isUnknownEntityError(final AbstractOXException e) {
-        return EnumComponent.ACL_ERROR.equals(e.getComponent()) && Entity2ACLException.Code.RESOLVE_USER_FAILED.getNumber() == e.getDetailNumber();
+        final Component component = e.getComponent();
+        final int detailNumber = e.getDetailNumber();
+        return (EnumComponent.ACL_ERROR.equals(component) && (Entity2ACLException.Code.RESOLVE_USER_FAILED.getNumber() == detailNumber)) || (EnumComponent.USER.equals(component) && (LdapException.Code.USER_NOT_FOUND.getDetailNumber() == detailNumber));
     }
 
     private static boolean checkForNamespaceFolder(final String fullname, final IMAPStore imapStore, final Session session, final int accountId) throws MessagingException {
@@ -642,7 +664,46 @@ public final class IMAPFolderConverter {
         return false;
     }
 
-    private static Rights getOwnRightsInternal(final IMAPFolder folder, final Session session, final IMAPConfig imapConfig) {
+    /**
+     * Gets the unread count from given IMAP folder.
+     * 
+     * @param imapFolder The IMAP folder
+     * @return The unread count
+     * @throws MailException If returning unread count fails
+     */
+    public static int getUnreadCount(final IMAPFolder imapFolder) throws MailException {
+        try {
+            try {
+                return imapFolder.getUnreadMessageCount();
+            } catch (final MessagingException e) {
+                final Exception nested = e.getNextException();
+                if (nested instanceof ParsingException && nested.getMessage().indexOf("STATUS") != -1) {
+                    /*
+                     * Parsing of STATUS response failed
+                     */
+                    final int[] status = IMAPCommandsCollection.getStatus(imapFolder);
+                    return status[2];
+                }
+                // Re-throw
+                throw e;
+            }
+        } catch (final MessagingException e) {
+            throw MIMEMailException.handleMessagingException(e);
+        }
+    }
+
+    /**
+     * Gets the session user's own rights.
+     * <p>
+     * <b>Note</b>: This method assumes all preconditions were met (exists, selectable, etc.) to perform MYRIGHTS command on specified IMAP
+     * folder.
+     * 
+     * @param folder The IMAP folder
+     * @param session The session
+     * @param imapConfig The IMAP configuration
+     * @return The own rights
+     */
+    public static Rights getOwnRights(final IMAPFolder folder, final Session session, final IMAPConfig imapConfig) {
         if (folder instanceof DefaultFolder) {
             return null;
         }
@@ -651,14 +712,12 @@ public final class IMAPFolderConverter {
             try {
                 retval = RightsCache.getCachedRights(folder, true, session, imapConfig.getAccountId());
             } catch (final MessagingException e) {
-                if ((e.getNextException() instanceof com.sun.mail.iap.CommandFailedException) && (e.getNextException().getMessage().indexOf(
-                    "NO Mailbox does not exist") != -1)) {
+                final Exception nextException = e.getNextException();
+                if ((nextException instanceof com.sun.mail.iap.CommandFailedException)) {
                     /*
-                     * This occurs when requesting MYRIGHTS on a shared folder. Just log a warning!
+                     * Handle command failed exception
                      */
-                    if (LOG.isWarnEnabled()) {
-                        LOG.warn(IMAPException.getFormattedMessage(IMAPException.Code.FOLDER_NOT_FOUND, folder.getFullName()), e);
-                    }
+                    handleCommandFailedException(((com.sun.mail.iap.CommandFailedException) nextException), folder.getFullName());
                 } else {
                     LOG.error(e.getMessage(), e);
                 }
@@ -687,6 +746,20 @@ public final class IMAPFolderConverter {
         return retval;
     }
 
+    private static void handleCommandFailedException(final com.sun.mail.iap.CommandFailedException e, final String fullname) {
+        final String msg = e.getMessage().toLowerCase(Locale.ENGLISH);
+        if (msg.indexOf("Mailbox doesn't exist") >= 0 || msg.indexOf("Mailbox does not exist") >= 0) {
+            /*
+             * This occurs when requesting MYRIGHTS on a shared or somehow non-existent folder. Just log a warning!
+             */
+            if (LOG.isWarnEnabled()) {
+                LOG.warn(IMAPException.getFormattedMessage(IMAPException.Code.FOLDER_NOT_FOUND, fullname), e);
+            }
+        } else {
+            LOG.error(e.getMessage(), e);
+        }
+    }
+
     /**
      * Simple proxy class for {@link ACLExtension} to have it loaded only when needed.
      */
